import { defineStore } from 'pinia';
import { ref } from 'vue';
import { apiClient, API_ENDPOINTS } from '@/services/api';
import type { TimeEntry, TimeEntryCreate, TimeEntryUpdate } from '@/types';

export interface TimeEntry {
    id: number;
    chore_assignment_id: number;
    user_id: number;
    start_time: string;
    end_time?: string | null;
    duration_seconds?: number | null;
}

export const useTimeEntryStore = defineStore('timeEntry', () => {
    const timeEntries = ref<Record<number, TimeEntry[]>>({});
    const loading = ref(false);
    const error = ref<string | null>(null);

    async function fetchTimeEntriesForAssignment(assignmentId: number) {
        loading.value = true;
        error.value = null;
        try {
            const response = await apiClient.get(`${API_ENDPOINTS.CHORES.ASSIGNMENT_BY_ID(assignmentId)}/time-entries`);
            timeEntries.value[assignmentId] = response.data;
        } catch (e) {
            error.value = 'Failed to fetch time entries.';
            console.error(e);
        } finally {
            loading.value = false;
        }
    }

    async function startTimeEntry(assignmentId: number) {
        loading.value = true;
        error.value = null;
        try {
            const response = await apiClient.post(`${API_ENDPOINTS.CHORES.ASSIGNMENT_BY_ID(assignmentId)}/time-entries`);
            if (!timeEntries.value[assignmentId]) {
                timeEntries.value[assignmentId] = [];
            }
            timeEntries.value[assignmentId].push(response.data);
        } catch (e) {
            error.value = 'Failed to start time entry.';
            console.error(e);
        } finally {
            loading.value = false;
        }
    }

    async function stopTimeEntry(assignmentId: number, timeEntryId: number) {
        loading.value = true;
        error.value = null;
        try {
            const response = await apiClient.put(`${API_ENDPOINTS.CHORES.TIME_ENTRY(timeEntryId)}`);
            const assignmentEntries = timeEntries.value[assignmentId];
            if (assignmentEntries) {
                const index = assignmentEntries.findIndex(te => te.id === timeEntryId);
                if (index !== -1) {
                    assignmentEntries[index] = response.data;
                }
            }
        } catch (e) {
            error.value = 'Failed to stop time entry.';
            console.error(e);
        } finally {
            loading.value = false;
        }
    }

    return {
        timeEntries,
        loading,
        error,
        fetchTimeEntriesForAssignment,
        startTimeEntry,
        stopTimeEntry,
    };
}); 